import argparse
import csv
import sys
import time
from os import environ
from dateutil.parser import parse
import twitter

__author__ = 'Koen Rouwhorst'
__version__ = '0.1'


class TwitterClient(object):
    def __init__(self,
                 consumer_key,
                 consumer_secret,
                 access_token_key,
                 access_token_secret):
        self.consumer_key = consumer_key
        self.consumer_secret = consumer_secret
        self.access_token_key = access_token_key
        self.access_token_secret = access_token_secret

    def delete_tweets(self, date, limit):
        """Delete all tweets that match the specified criteria."""

        client = twitter.Api(consumer_key=self.consumer_key,
                             consumer_secret=self.consumer_secret,
                             access_token_key=self.access_token_key,
                             access_token_secret=self.access_token_secret)

        with open('tweets.csv') as csv_file:
            count = 0

            for row in csv.DictReader(csv_file):
                tweet_id = int(row['tweet_id'])
                tweet_date = parse(row['timestamp'], ignoretz=True).date()

                if date and tweet_date >= parse(date).date():
                    continue

                if (limit == 'retweet'
                        and not row['retweeted_status_id']
                        or limit == 'reply'
                        and not row['in_reply_to_status_id']):
                    continue

                try:
                    print("Deleting tweet #{0} ({1})").format(tweet_id,
                                                              tweet_date)

                    client.DestroyStatus(tweet_id)
                    count += 1
                    time.sleep(0.2)

                except twitter.error.TwitterError as exception:
                    sys.stderr.write("Twitter error: %s\n" % exception.message)

                except csv.Error as exception:
                    sys.stderr.write("CSV error: %s\n" % exception.message)
                    exit(1)

        print "Number of deleted tweets: %s\n" % count


def main():
    parser = argparse.ArgumentParser(description='Delete old tweets.')
    parser.add_argument('--consumer-key',
                        help='Twitter consumer key. Use this or set the'
                             'environment variable TWITTER_CONSUMER_KEY.',
                        default=environ.get('TWITTER_CONSUMER_KEY'))
    parser.add_argument('--consumer-secret',
                        help='Twitter consumer secret. Use this or set the'
                             'environment variable TWITTER_CONSUMER_SECRET.',
                        default=environ.get('TWITTER_CONSUMER_SECRET'))
    parser.add_argument('--access-token',
                        help='Twitter access token. Use this or set the'
                             'environment variable TWITTER_ACCESS_TOKEN.',
                        default=environ.get('TWITTER_ACCESS_TOKEN'))
    parser.add_argument('--access-token-secret',
                        help='Twitter access token secret. Use this or set '
                             'environment variable '
                             'TWITTER_ACCESS_TOKEN_SECRET.',
                        default=environ.get('TWITTER_ACCESS_TOKEN_SECRET'))
    parser.add_argument('--date', '-d',
                        dest='date',
                        required=True,
                        help='Delete tweets until this date')
    parser.add_argument('--limit', '-l',
                        dest='limit',
                        choices=['reply', 'retweet'],
                        help='Limit to either replies or retweets')

    args = parser.parse_args()

    if args.consumer_key is None or args.consumer_secret is None:
        sys.stderr.write('No consumer key or secret set.')
        exit(1)

    if args.access_token is None or args.access_token_secret is None:
        sys.stderr.write('No access token or secret set.')
        exit(1)

    client = TwitterClient(args.consumer_key,
                           args.consumer_secret,
                           args.access_token,
                           args.access_token_secret)

    client.delete_tweets(args.date, args.limit)

if __name__ == '__main__':
    main()
